#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import copy
from yagocd.session import Session
from yagocd.resources.agent import AgentManager
from yagocd.resources.artifact import ArtifactManager
from yagocd.resources.configuration import ConfigurationManager
from yagocd.resources.feed import FeedManager
from yagocd.resources.job import JobManager
from yagocd.resources.material import MaterialManager
from yagocd.resources.pipeline import PipelineManager
from yagocd.resources.property import PropertyManager
from yagocd.resources.stage import StageManager
from yagocd.resources.user import UserManager


class Yagocd(object):
    """
    Main class of the package, that gives user access to Go REST API methods.
    """

    DEFAULT_OPTIONS = {
        'server': 'http://localhost:8153',
        'context_path': 'go/',
        'api_path': 'api/',
        'verify': True,
        'headers': {
            'Accept': 'application/vnd.go.cd.v1+json',
        }
    }

    def __init__(self, server=None, auth=None, options=None):
        """
        Construct a GOCD client instance.

        :param server: url of the Go server
        :param auth: authorization, that will be passed to requests.
        Could tuple of (username, password) for basic authentication.
        :param options: dictionary of additional options.
            * context_path -- server context path to use (default is ``go/``)
            * api_path -- api endpoint to use. By default ``api/`` will be used, but in some cases this will be
            overwritten by some managers, because of API.
            * verify -- verify SSL certs. Defaults to ``True``.
            * headers -- default headers for requests (default is ``'Accept': 'application/vnd.go.cd.v1+json'``)
        """
        options = {} if options is None else options

        if server is not None:
            options['server'] = server

        merged = copy.deepcopy(self.DEFAULT_OPTIONS)
        merged.update(options)

        self._session = Session(auth, merged)

    @property
    def server_url(self):
        """
        Property for getting server url.
        :return: server url for this instance.
        """
        return self._session.server_url

    @property
    def agents(self):
        """
        Property for accessing ``AgentManager`` instance, which is used to manage agents.

        :rtype: yagocd.resources.agent.AgentManager
        """
        return AgentManager(session=self._session)

    @property
    def artifacts(self):
        """
        Property for accessing ``ArtifactManager`` instance, which is used to manage artifacts.

        :rtype: yagocd.resources.artifact.ArtifactManager
        """
        return ArtifactManager(session=self._session)

    @property
    def configurations(self):
        """
        Property for accessing ``ConfigurationManager`` instance, which is used to manage configurations.

        :rtype: yagocd.resources.configuration.ConfigurationManager
        """
        return ConfigurationManager(session=self._session)

    @property
    def feeds(self):
        """
        Property for accessing ``FeedManager`` instance, which is used to manage feeds.

        :rtype: yagocd.resources.feed.FeedManager
        """
        return FeedManager(session=self._session)

    @property
    def jobs(self):
        """
        Property for accessing ``JobManager`` instance, which is used to manage feeds.

        :rtype: yagocd.resources.job.JobManager
        """
        return JobManager(session=self._session)

    @property
    def materials(self):
        """
        Property for accessing ``MaterialManager`` instance, which is used to manage materials.

        :rtype: yagocd.resources.material.MaterialManager
        """
        return MaterialManager(session=self._session)

    @property
    def pipelines(self):
        """
        Property for accessing ``PipelineManager`` instance, which is used to manage pipelines.

        :rtype: yagocd.resources.pipeline.PipelineManager
        """
        return PipelineManager(session=self._session)

    @property
    def properties(self):
        """
        Property for accessing ``PropertyManager`` instance, which is used to manage properties of the jobs.

        :rtype: yagocd.resources.property.PropertyManager
        """
        return PropertyManager(session=self._session)

    @property
    def stages(self):
        """
        Property for accessing ``StageManager`` instance, which is used to manage stages.

        :rtype: yagocd.resources.stage.StageManager
        """
        return StageManager(session=self._session)

    @property
    def users(self):
        """
        Property for accessing ``UserManager`` instance, which is used to manage users.

        :rtype: yagocd.resources.user.UserManager
        """
        return UserManager(session=self._session)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd import Yagocd
from yagocd.resources import pipeline, stage, job
from yagocd.session import Session

import mock
import pytest


class TestStageInstance(object):
    @pytest.fixture()
    def session(self):
        return Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)

    @pytest.fixture()
    def stage_instance_from_pipeline(self, my_vcr, session):
        with my_vcr.use_cassette("stage/stage_instance_from_pipeline"):
            return pipeline.PipelineManager(session).find('Consumer_Website').history()[0].stages()[0]

    @pytest.fixture()
    def stage_instance_from_stage_history(self, my_vcr, session):
        with my_vcr.use_cassette("stage/stage_instance_from_stage_history"):
            return stage.StageManager(session).history('Consumer_Website', 'Commit')[0]

    def test_has_all_managers_methods(self):
        excludes = ['get', 'history', 'full_history']

        def get_public_methods(klass):
            methods = set()
            for name in dir(klass):
                if name.startswith('_'):
                    continue

                candidate = getattr(klass, name)
                if hasattr(candidate, '__call__'):
                    methods.add(name)
            return methods

        managers_methods = get_public_methods(stage.StageManager)
        entity_methods = get_public_methods(stage.StageInstance)
        result = managers_methods - entity_methods - set(excludes)
        assert len(result) == 0, "Some methods are missing in pipeline entity: {}".format(result)

    def test_url_from_pipeline(self, stage_instance_from_pipeline):
        assert stage_instance_from_pipeline.url == 'http://localhost:8153/go/pipelines/Consumer_Website/29/Commit/1'

    def test_url_from_history(self, stage_instance_from_stage_history):
        assert stage_instance_from_stage_history.url == 'http://localhost:8153/go/pipelines/Consumer_Website/29/Commit/1'

    def test_pipeline_is_not_none(self, stage_instance_from_pipeline):
        assert stage_instance_from_pipeline.pipeline is not None

    def test_pipeline_is_none(self, stage_instance_from_stage_history):
        assert stage_instance_from_stage_history.pipeline is None

    def test_stage_counter_from_pipeline(self, stage_instance_from_pipeline):
        assert stage_instance_from_pipeline.stage_counter == '1'

    def test_stage_counter_from_stage(self, stage_instance_from_stage_history):
        assert stage_instance_from_stage_history.stage_counter == '1'

    @mock.patch('yagocd.resources.stage.StageManager.cancel')
    def test_cancel_call_from_pipeline(self, cancel_mock, stage_instance_from_pipeline):
        stage_instance_from_pipeline.cancel()
        cancel_mock.assert_called_with(
            pipeline_name=stage_instance_from_pipeline.pipeline_name,
            stage_name=stage_instance_from_pipeline.stage_name
        )

    @mock.patch('yagocd.resources.stage.StageManager.cancel')
    def test_cancel_call_from_stage(self, cancel_mock, stage_instance_from_stage_history):
        stage_instance_from_stage_history.cancel()
        cancel_mock.assert_called_with(
            pipeline_name=stage_instance_from_stage_history.pipeline_name,
            stage_name=stage_instance_from_stage_history.stage_name
        )

    def test_jobs_return_type_from_pipeline(self, stage_instance_from_pipeline):
        jobs = stage_instance_from_pipeline.jobs()
        assert all(isinstance(j, job.JobInstance) for j in jobs)

    def test_jobs_return_type_from_stage(self, stage_instance_from_stage_history):
        jobs = stage_instance_from_stage_history.jobs()
        assert all(isinstance(j, job.JobInstance) for j in jobs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.resources.job import JobInstance
from yagocd.resources import BaseManager, Base


class StageManager(BaseManager):
    """
    The stages API allows users to view stage information and operate on it.
    """

    def __init__(
        self,
        session,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None,
    ):
        super(StageManager, self).__init__(session)

        self._pipeline_name = pipeline_name
        self._pipeline_counter = pipeline_counter
        self._stage_name = stage_name
        self._stage_counter = stage_counter

    def cancel(self, pipeline_name=None, stage_name=None):
        """
        Cancel an active stage of a specified stage.

        :param pipeline_name: pipeline name.
        :param stage_name: stage name.
        :return: a text confirmation.
        """
        assert self._pipeline_name or pipeline_name
        assert self._stage_name or stage_name

        response = self._session.post(
            path='{base_api}/stages/{pipeline_name}/{stage_name}/cancel'.format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                stage_name=self._stage_name or stage_name
            ),
            headers={'Accept': 'application/json'},
        )
        return response.text

    def get(
        self,
        pipeline_name=None,
        pipeline_counter=None,
        stage_name=None,
        stage_counter=None
    ):
        """
        Gets stage instance object.

        :param pipeline_name: pipeline name.
        :param stage_name: stage name.
        :param pipeline_counter: pipeline counter.
        :param stage_counter: stage counter.
        :return: a stage instance object :class:`yagocd.resources.stage.StageInstance`.
        :rtype: yagocd.resources.stage.StageInstance
        """
        assert self._pipeline_name or pipeline_name
        assert self._pipeline_counter or pipeline_counter
        assert self._stage_name or stage_name
        assert self._stage_counter or stage_counter

        response = self._session.get(
            path='{base_api}/stages/{pipeline_name}/{stage_name}/instance/{pipeline_counter}/{stage_counter}'.format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                pipeline_counter=self._pipeline_counter or pipeline_counter,
                stage_name=self._stage_name or stage_name,
                stage_counter=self._stage_counter or stage_counter
            ),
            headers={'Accept': 'application/json'},
        )

        return StageInstance(session=self._session, data=response.json(), pipeline=None)

    def history(self, pipeline_name=None, stage_name=None, offset=0):
        """
        The stage history allows users to list stage instances of specified stage.
        Supports pagination using offset which tells the API how many instances to skip.

        :param pipeline_name: pipeline name.
        :param stage_name: stage name.
        :param offset: how many instances to skip.
        :return: an array of stage instances :class:`yagocd.resources.stage.StageInstance`.
        :rtype: list of yagocd.resources.stage.StageInstance
        """
        assert self._pipeline_name or pipeline_name
        assert self._stage_name or stage_name

        response = self._session.get(
            path='{base_api}/stages/{pipeline_name}/{stage_name}/history/{offset}'.format(
                base_api=self.base_api,
                pipeline_name=self._pipeline_name or pipeline_name,
                stage_name=self._stage_name or stage_name,
                offset=offset,
            ),
            headers={'Accept': 'application/json'},
        )

        instances = list()
        for instance in response.json().get('stages'):
            instances.append(StageInstance(session=self._session, data=instance, pipeline=None))

        return instances

    def full_history(self, pipeline_name=None, stage_name=None):
        """
        The stage history allows users to list stage instances of specified stage.

        This method uses generator to get full stage history.
        :param pipeline_name: pipeline name.
        :param stage_name: stage name.
        :return: an array of stage instances :class:`yagocd.resources.stage.StageInstance`.
        :rtype: list of yagocd.resources.stage.StageInstance
        """
        offset = 0
        instances = self.history(pipeline_name, stage_name, offset)
        while instances:
            for instance in instances:
                yield instance

            offset += len(instances)
            instances = self.history(pipeline_name, stage_name, offset)


class StageInstance(Base):
    """
    Class representing instance of specific stage.
    """

    def __init__(self, session, data, pipeline):
        super(StageInstance, self).__init__(session, data)
        self._pipeline = pipeline

        self._manager = StageManager(session=self._session)

    @property
    def url(self):
        """
        Returns url for accessing stage instance.
        """
        return "{server_url}/go/pipelines/{pipeline_name}/{pipeline_counter}/{stage_name}/{stage_counter}".format(
            server_url=self._session.server_url,
            pipeline_name=self.pipeline_name,
            pipeline_counter=self.pipeline_counter,
            stage_name=self.data.name,
            stage_counter=self.data.counter,
        )

    @property
    def pipeline_name(self):
        """
        Get pipeline name of current stage instance.

        Because instantiating stage instance could be performed in different ways and those return different results,
        we have to check where from to get name of the pipeline.

        :return: pipeline name.
        """
        if 'pipeline_name' in self.data:
            return self.data.get('pipeline_name')
        elif self.pipeline is not None:
            return self.pipeline.data.name

    @property
    def pipeline_counter(self):
        """
        Get pipeline counter of current stage instance.

        Because instantiating stage instance could be performed in different ways and those return different results,
        we have to check where from to get counter of the pipeline.

        :return: pipeline counter.
        """
        if 'pipeline_counter' in self.data:
            return self.data.get('pipeline_counter')
        elif self.pipeline is not None:
            return self.pipeline.data.counter

    @property
    def stage_name(self):
        """
        Get stage name of current instance.

        This method is to be inline with others.

        :return: stage name.
        """
        if 'name' in self.data:
            return self.data.get('name')

    @property
    def stage_counter(self):
        """
        Get stage counter of current instance.

        This method is to be inline with others.

        :return: stage counter.
        """
        if 'counter' in self.data:
            return self.data.get('counter')

    @property
    def pipeline(self):
        return self._pipeline

    def cancel(self):
        """
        Cancel an active stage of a specified stage.

        :return: a text confirmation.
        """
        return self._manager.cancel(pipeline_name=self.pipeline_name, stage_name=self.stage_name)

    def jobs(self):
        """
        Method for getting jobs from stage instance.

        :return: arrays of jobs.
        :rtype: list of yagocd.resources.job.JobInstance
        """
        jobs = list()
        for data in self.data.jobs:
            jobs.append(JobInstance(session=self._session, data=data, stage=self))

        return jobs



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from six.moves.urllib.parse import urlencode

from yagocd.client import Yagocd
from yagocd.session import Session
from yagocd.resources import material

import pytest


class BaseTestConfigurationManager(object):
    @pytest.fixture()
    def session(self):
        return Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)

    @pytest.fixture()
    def manager(self, session):
        return material.MaterialManager(session=session)


class TestList(BaseTestConfigurationManager):
    def test_list_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("material/list") as cass:
            manager.list()
            assert cass.requests[0].path == '/go/api/config/materials'

    def test_list_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("material/list") as cass:
            manager.list()
            assert cass.requests[0].method == 'GET'

    def test_list_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("material/list") as cass:
            manager.list()
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_list_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("material/list") as cass:
            manager.list()
            assert cass.responses[0]['status']['code'] == 200

    def test_list_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("material/list"):
            result = manager.list()
            assert all(isinstance(i, material.MaterialEntity) for i in result)


class TestModifications(BaseTestConfigurationManager):
    FINGERPRINT = 'e302ce7f43cd1a5009d218e7d6c1adf6a38fa9a33f6d0054d1607a00209fa810'

    def test_modifications_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("material/modifications") as cass:
            manager.modifications(self.FINGERPRINT)
            assert cass.requests[0].path == '/go/api/materials/{0}/modifications/{1}'.format(
                self.FINGERPRINT, 0
            )

    def test_modifications_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("material/modifications") as cass:
            manager.modifications(self.FINGERPRINT)
            assert cass.requests[0].method == 'GET'

    def test_modifications_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("material/modifications") as cass:
            manager.modifications(self.FINGERPRINT)
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_modifications_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("material/modifications") as cass:
            manager.modifications(self.FINGERPRINT)
            assert cass.responses[0]['status']['code'] == 200

    def test_modifications_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("material/modifications"):
            result = manager.modifications(self.FINGERPRINT)
            assert all(isinstance(i, material.ModificationEntity) for i in result)


#
# class TestNotifySvn(BaseTestConfigurationManager):
#     UUID = ''
#
#     def test_notify_svn_request_url(self, manager, my_vcr):
#         with my_vcr.use_cassette("material/notify_svn") as cass:
#             manager.notify_svn(self.UUID)
#             assert cass.requests[0].path == '/material/notify/svn'
#
#     def test_notify_svn_request_method(self, manager, my_vcr):
#         with my_vcr.use_cassette("material/notify_svn") as cass:
#             manager.notify_svn(self.UUID)
#             assert cass.requests[0].method == 'POST'
#
#     def test_notify_svn_request_data(self, manager, my_vcr):
#         with my_vcr.use_cassette("material/notify_svn") as cass:
#             manager.notify_svn(self.UUID)
#             assert False  # add check for uuid in data
#
#     def test_notify_svn_request_accept_headers(self, manager, my_vcr):
#         with my_vcr.use_cassette("material/notify_svn") as cass:
#             manager.notify_svn(self.UUID)
#             assert cass.requests[0].headers['accept'] == 'application/json'
#
#     def test_notify_svn_response_code(self, manager, my_vcr):
#         with my_vcr.use_cassette("material/notify_svn") as cass:
#             manager.notify_svn(self.UUID)
#             assert cass.responses[0]['status']['code'] == 200
#
#     def test_notify_svn_return_type(self, manager, my_vcr):
#         with my_vcr.use_cassette("material/notify_svn"):
#             result = manager.notify_svn(self.UUID)
#             assert all(isinstance(i, material.ModificationEntity) for i in result)


class TestNotifyGit(BaseTestConfigurationManager):
    URL = 'https://github.com/gocd-demo/services.git'

    def test_notify_git_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("material/notify_git") as cass:
            manager.notify_git(self.URL)
            assert cass.requests[0].path == '/go/api/material/notify/git'

    def test_notify_git_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("material/notify_git") as cass:
            manager.notify_git(self.URL)
            assert cass.requests[0].method == 'POST'

    def test_notify_git_request_data(self, manager, my_vcr):
        with my_vcr.use_cassette("material/notify_git") as cass:
            manager.notify_git(self.URL)
            assert cass.requests[0].body.decode('ascii') == urlencode({'repository_url': self.URL})

    def test_notify_git_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("material/notify_git") as cass:
            manager.notify_git(self.URL)
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_notify_git_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("material/notify_git") as cass:
            manager.notify_git(self.URL)
            assert cass.responses[0]['status']['code'] == 202

    def test_notify_git_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("material/notify_git"):
            result = manager.notify_git(self.URL)
            assert result == ('The material is now scheduled for an update. '
                              'Please check relevant pipeline(s) for status.\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.client import Yagocd
from yagocd.session import Session
from yagocd.resources import stage

import pytest
import mock
from six import string_types


class BaseTestStageManager(object):
    PIPELINE_NAME = 'Shared_Services'
    PIPELINE_COUNTER = 7
    STAGE_NAME = 'Commit'
    STAGE_COUNTER = '1'

    @pytest.fixture()
    def session(self):
        return Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)

    @pytest.fixture()
    def manager(self, session):
        return stage.StageManager(
            session=session,
            pipeline_name=self.PIPELINE_NAME,
            pipeline_counter=self.PIPELINE_COUNTER,
            stage_name=self.STAGE_NAME,
            stage_counter=self.STAGE_COUNTER
        )


class TestCancel(BaseTestStageManager):
    def test_cancel_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_cancel") as cass:
            manager.cancel()
            assert cass.requests[0].path == '/go/api/stages/{pipeline_name}/{stage_name}/cancel'.format(
                pipeline_name=self.PIPELINE_NAME,
                stage_name=self.STAGE_NAME
            )

    def test_cancel_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_cancel") as cass:
            manager.cancel()
            assert cass.requests[0].method == 'POST'

    def test_cancel_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_cancel") as cass:
            manager.cancel()
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_cancel_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_cancel") as cass:
            manager.cancel()
            assert cass.responses[0]['status']['code'] == 200

    def test_cancel_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_cancel"):
            result = manager.cancel()
            assert isinstance(result, string_types)

    def test_cancel_return_value(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_cancel"):
            result = manager.cancel()
            assert result == 'Stage cancelled successfully.\n'


class TestGet(BaseTestStageManager):
    def test_get_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_get") as cass:
            manager.get()
            assert cass.requests[0].path == (
                '/go'
                '/api'
                '/stages'
                '/{pipeline_name}'
                '/{stage_name}'
                '/instance'
                '/{pipeline_counter}'
                '/{stage_counter}'
            ).format(
                pipeline_name=self.PIPELINE_NAME,
                stage_name=self.STAGE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_counter=self.STAGE_COUNTER
            )

    def test_get_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_get") as cass:
            manager.get()
            assert cass.requests[0].method == 'GET'

    def test_get_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_get") as cass:
            manager.get()
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_get_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_get") as cass:
            manager.get()
            assert cass.responses[0]['status']['code'] == 200

    def test_get_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_get"):
            result = manager.get()
            assert isinstance(result, stage.StageInstance)


class TestHistory(BaseTestStageManager):
    def test_history_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_history") as cass:
            manager.history()
            assert cass.requests[0].path == '/go/api/stages/{pipeline_name}/{stage_name}/history/{offset}'.format(
                pipeline_name=self.PIPELINE_NAME,
                stage_name=self.STAGE_NAME,
                offset=0
            )

    def test_history_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_history") as cass:
            manager.history()
            assert cass.requests[0].method == 'GET'

    def test_history_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_history") as cass:
            manager.history()
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_history_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_history") as cass:
            manager.history()
            assert cass.responses[0]['status']['code'] == 200

    def test_history_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("stage/stage_history"):
            result = manager.history()
            assert all(isinstance(s, stage.StageInstance) for s in result)


class TestFullHistory(BaseTestStageManager):
    @mock.patch('yagocd.resources.stage.StageManager.history')
    def test_history_is_called(self, history_mock, manager):
        history_mock.side_effect = [['foo', 'bar', 'baz'], []]

        list(manager.full_history(self.PIPELINE_NAME, self.STAGE_NAME))

        calls = [mock.call(self.PIPELINE_NAME, self.STAGE_NAME, 0), mock.call(self.PIPELINE_NAME, self.STAGE_NAME, 3)]
        history_mock.assert_has_calls(calls)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import pytest
from six import string_types

from yagocd.client import Yagocd
from yagocd.session import Session
from yagocd.resources import job


class BaseTestJobManager(object):
    @pytest.fixture()
    def session(self):
        return Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)

    @pytest.fixture()
    def manager(self, session):
        return job.JobManager(session=session)


class TestScheduled(BaseTestJobManager):
    def test_scheduled_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("job/scheduled") as cass:
            manager.scheduled()
            assert cass.requests[0].path == '/go/api/jobs/scheduled.xml'

    def test_scheduled_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("job/scheduled") as cass:
            manager.scheduled()
            assert cass.requests[0].method == 'GET'

    def test_scheduled_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("job/scheduled") as cass:
            manager.scheduled()
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_scheduled_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("job/scheduled") as cass:
            manager.scheduled()
            assert cass.responses[0]['status']['code'] == 200

    def test_scheduled_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("job/scheduled"):
            result = manager.scheduled()
            assert isinstance(result, string_types)


class TestHistory(BaseTestJobManager):
    PIPELINE_NAME = 'Shared_Services'
    STAGE_NAME = 'Commit'
    JOB_NAME = 'build'

    def test_history_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("job/history") as cass:
            manager.history(
                pipeline_name=self.PIPELINE_NAME,
                stage_name=self.STAGE_NAME,
                job_name=self.JOB_NAME
            )
            assert cass.requests[0].path == '/go/api/jobs/{0}/{1}/{2}/history/{3}'.format(
                self.PIPELINE_NAME, self.STAGE_NAME, self.JOB_NAME, 0
            )

    def test_history_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("job/history") as cass:
            manager.history(
                pipeline_name=self.PIPELINE_NAME,
                stage_name=self.STAGE_NAME,
                job_name=self.JOB_NAME
            )
            assert cass.requests[0].method == 'GET'

    def test_history_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("job/history") as cass:
            manager.history(
                pipeline_name=self.PIPELINE_NAME,
                stage_name=self.STAGE_NAME,
                job_name=self.JOB_NAME
            )
            assert cass.requests[0].headers['accept'] == 'application/xml'

    def test_history_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("job/history") as cass:
            manager.history(
                pipeline_name=self.PIPELINE_NAME,
                stage_name=self.STAGE_NAME,
                job_name=self.JOB_NAME
            )
            assert cass.responses[0]['status']['code'] == 200

    def test_history_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("job/history"):
            result = manager.history(
                pipeline_name=self.PIPELINE_NAME,
                stage_name=self.STAGE_NAME,
                job_name=self.JOB_NAME
            )
            assert all(isinstance(i, job.JobInstance) for i in result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.client import Yagocd
from yagocd.session import Session
from yagocd.resources import property

import pytest
from six import string_types
from six.moves.urllib.parse import urlencode


class BaseTestPropertyManager(object):
    PIPELINE_NAME = 'Shared_Services'
    PIPELINE_COUNTER = 7
    STAGE_NAME = 'Commit'
    STAGE_COUNTER = '1'
    JOB_NAME = 'build'

    @pytest.fixture()
    def session(self):
        return Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)

    @pytest.fixture()
    def manager(self, session):
        return property.PropertyManager(
            session=session,
            pipeline_name=self.PIPELINE_NAME,
            pipeline_counter=self.PIPELINE_COUNTER,
            stage_name=self.STAGE_NAME,
            stage_counter=self.STAGE_COUNTER,
            job_name=self.JOB_NAME
        )


class TestList(BaseTestPropertyManager):
    def test_list_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_list") as cass:
            manager.list()
            assert cass.requests[0].path == (
                '/go'
                '/properties'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}'
            ).format(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER,
                job_name=self.JOB_NAME
            )

    def test_list_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_list") as cass:
            manager.list()
            assert cass.requests[0].method == 'GET'

    def test_list_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_list") as cass:
            manager.list()
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_list_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_list") as cass:
            manager.list()
            assert cass.responses[0]['status']['code'] == 200

    def test_list_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_list"):
            result = manager.list()
            assert isinstance(result, dict)


class TestGet(BaseTestPropertyManager):
    PROPERTY_NAME = 'cruise_pipeline_counter'

    def test_get_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_get") as cass:
            manager.get(self.PROPERTY_NAME)
            assert cass.requests[0].path == (
                '/go'
                '/properties'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}'
                '/{property_name}'
            ).format(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER,
                job_name=self.JOB_NAME,
                property_name=self.PROPERTY_NAME
            )

    def test_get_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_get") as cass:
            manager.get(self.PROPERTY_NAME)
            assert cass.requests[0].method == 'GET'

    def test_get_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_get") as cass:
            manager.get(self.PROPERTY_NAME)
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_get_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_get") as cass:
            manager.get(self.PROPERTY_NAME)
            assert cass.responses[0]['status']['code'] == 200

    def test_get_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_get"):
            result = manager.get(self.PROPERTY_NAME)
            assert isinstance(result, dict)


class TestHistorical(BaseTestPropertyManager):
    def test_historical_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_historical") as cass:
            manager.historical()
            assert cass.requests[0].path == '/go/properties/search'

    def test_historical_request_params(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_historical") as cass:
            manager.historical()
            assert cass.requests[0].query == [
                ('jobName', self.JOB_NAME),
                ('pipelineName', self.PIPELINE_NAME),
                ('stageName', self.STAGE_NAME)
            ]

    def test_historical_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_historical") as cass:
            manager.historical()
            assert cass.requests[0].method == 'GET'

    def test_historical_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_historical") as cass:
            manager.historical()


class TestCreate(BaseTestPropertyManager):
    PROPERTY_NAME = 'foo_bar'
    PROPERTY_VALUE = 100500

    def test_create_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_create") as cass:
            manager.create(self.PROPERTY_NAME, self.PROPERTY_VALUE)
            assert cass.requests[0].path == (
                '/go'
                '/properties'
                '/{pipeline_name}'
                '/{pipeline_counter}'
                '/{stage_name}'
                '/{stage_counter}'
                '/{job_name}'
                '/{property_name}'
            ).format(
                pipeline_name=self.PIPELINE_NAME,
                pipeline_counter=self.PIPELINE_COUNTER,
                stage_name=self.STAGE_NAME,
                stage_counter=self.STAGE_COUNTER,
                job_name=self.JOB_NAME,
                property_name=self.PROPERTY_NAME
            )

    def test_create_request_params(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_create") as cass:
            manager.create(self.PROPERTY_NAME, self.PROPERTY_VALUE)
            assert cass.requests[0].body.decode('ascii') == urlencode({'value': self.PROPERTY_VALUE})

    def test_create_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_create") as cass:
            manager.create(self.PROPERTY_NAME, self.PROPERTY_VALUE)
            assert cass.requests[0].method == 'POST'

    def test_create_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_create") as cass:
            manager.create(self.PROPERTY_NAME, self.PROPERTY_VALUE)
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_create_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_create") as cass:
            manager.create(self.PROPERTY_NAME, self.PROPERTY_VALUE)
            assert cass.responses[0]['status']['code'] == 201

    def test_create_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("property/property_create"):
            result = manager.create(self.PROPERTY_NAME, self.PROPERTY_VALUE)
            assert isinstance(result, string_types)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import time
import json
from collections import deque

from yagocd.resources import BaseManager, Base
from yagocd.resources.stage import StageInstance

from easydict import EasyDict


class PipelineManager(BaseManager):
    """
    The pipelines API allows users to view pipeline information and operate on it.
    """

    @staticmethod
    def tie_pipelines(pipelines):
        """
        Static method for tie-ing (linking) relevant pipelines.

        By default each pipeline object gives information about its dependencies, that are listed in materials. But
        you can't get its descendants, though it's possible. This method solves this problem.

        :param pipelines: list of pipelines.
        """
        for child in pipelines:
            parents = list()

            for parent in pipelines:
                children = list()
                candidates = [material for material in parent.data.materials if material.type == 'Pipeline']
                for child_candidate in candidates:
                    if child_candidate.description == child.data.name:
                        parents.append(parent)
                        children.append(child)

                parent.predecessors.extend(children)

            child.descendants = parents

    def list(self):
        """
        List all available pipelines.

        This method uses ``pipeline_groups`` API method call to list available pipelines.
        It also links them together, so later it's possible to refer to pipeline's descendants.
        :return: array of pipelines
        :rtype: list of yagocd.resources.pipeline.PipelineEntity
        """
        response = self._session.get(
            path='{base_api}/config/pipeline_groups'.format(base_api=self.base_api),
            headers={'Accept': 'application/json'},
        )

        pipelines = list()
        for group in response.json():
            for data in group['pipelines']:
                pipeline = PipelineEntity(
                    session=self._session,
                    data=data,
                    group=group['name']
                )
                pipelines.append(pipeline)

        # link descendants of each pipeline entity
        self.tie_pipelines(pipelines)

        return pipelines

    def find(self, name):
        """
        Finds pipeline by it's name.
        :param name: name of required pipeline.
        :return: if found - pipeline :class:`yagocd.resources.pipeline.PipelineEntity`, otherwise ``None``.
        :rtype: yagocd.resources.pipeline.PipelineEntity
        """
        for pipeline in self.list():
            if pipeline.data.name == name:
                return pipeline

    def history(self, name, offset=0):
        """
        The pipeline history allows users to list pipeline instances.
        Supports pagination using offset which tells the API how many instances to skip.

        :param name: name of the pipeline.
        :param offset: number of pipeline instances to be skipped.
        :return: an array of pipeline instances :class:`yagocd.resources.pipeline.PipelineInstance`.
        :rtype: list of yagocd.resources.pipeline.PipelineInstance
        """
        response = self._session.get(
            path='{base_api}/pipelines/{name}/history/{offset}'.format(
                base_api=self.base_api,
                name=name,
                offset=offset
            ),
            headers={'Accept': 'application/json'},
        )

        instances = list()
        for instance in response.json().get('pipelines'):
            instances.append(PipelineInstance(session=self._session, data=instance))

        return instances

    def full_history(self, name):
        """
        Method for accessing full history of specific pipeline.

        It yields each instance and after one chunk is over moves to the next one.
        :param name: name of the pipeline.
        :return: an array of pipeline instances :class:`yagocd.resources.pipeline.PipelineInstance`.
        :rtype: list of yagocd.resources.pipeline.PipelineInstance
        """
        offset = 0
        instances = self.history(name, offset)
        while instances:
            for instance in instances:
                yield instance

            offset += len(instances)
            instances = self.history(name, offset)

    def last(self, name):
        """
        Get last pipeline instance.

        :param name: name of the pipeline.
        :rtype: yagocd.resources.pipeline.PipelineInstance
        """
        pipeline_history = self.history(name=name)
        if pipeline_history:
            return pipeline_history[0]

    def get(self, name, counter):
        """
        Gets pipeline instance object.

        :param name: name of the pipeline.
        :param counter pipeline counter:
        :return: A pipeline instance object :class:`yagocd.resources.pipeline.PipelineInstance`.
        :rtype: yagocd.resources.pipeline.PipelineInstance
        """
        response = self._session.get(
            path='{base_api}/pipelines/{name}/instance/{counter}'.format(
                base_api=self.base_api,
                name=name,
                counter=counter
            ),
            headers={'Accept': 'application/json'},
        )

        return PipelineInstance(session=self._session, data=response.json())

    def status(self, name):
        """
        The pipeline status allows users to check if the pipeline is paused, locked and schedulable.

        :param name: name of the pipeline.
        :return: JSON containing information about pipeline state, wrapped in EasyDict class.
        """
        response = self._session.get(
            path='{base_api}/pipelines/{name}/status'.format(
                base_api=self.base_api,
                name=name,
            ),
            headers={'Accept': 'application/json'},
        )

        return EasyDict(response.json())

    def pause(self, name, cause):
        """
        Pause the specified pipeline.

        :param name: name of the pipeline.
        :param cause: reason for pausing the pipeline.
        """
        self._session.post(
            path='{base_api}/pipelines/{name}/pause'.format(
                base_api=self.base_api,
                name=name,
            ),
            data={'pauseCause': cause},
            headers={'Accept': 'application/json'},
        )

    def unpause(self, name):
        """
        Unpause the specified pipeline.

        :param name: name of the pipeline.
        """
        self._session.post(
            path='{base_api}/pipelines/{name}/unpause'.format(
                base_api=self.base_api,
                name=name,
            ),
            headers={'Accept': 'application/json'},
        )

    def release_lock(self, name):
        """
        Release a lock on a pipeline so that you can start up a new instance
        without having to wait for the earlier instance to finish.

        :param name: name of the pipeline.
        :return: a text confirmation.
        """
        response = self._session.post(
            path='{base_api}/pipelines/{name}/releaseLock'.format(
                base_api=self.base_api,
                name=name,
            ),
            headers={'Accept': 'application/json'},
        )
        return response.text

    def schedule(self, name, materials=None, variables=None, secure_variables=None):
        """
        Scheduling allows user to trigger a specific pipeline.

        :param name: name of the pipeline.
        :param materials: material revisions to use.
        :param variables: environment variables to set.
        :param secure_variables: secure environment variables to set.
        :return: a text confirmation.
        """

        data = {'materials': materials, 'variables': variables, 'secure_variables': secure_variables}
        data = dict((k, v) for k, v in data.items() if v is not None)

        response = self._session.post(
            path='{base_api}/pipelines/{name}/schedule'.format(
                base_api=self.base_api,
                name=name,
            ),
            data=json.dumps(data),
            headers={
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            },
        )

        return response.text

    def schedule_with_instance(
        self,
        name,
        materials=None,
        variables=None,
        secure_variables=None,
        backoff=0.5,
        max_tries=20
    ):
        """
        Schedule pipeline and return instance.
        Credits of implementation comes to `gaqzi`:
        https://github.com/gaqzi/py-gocd/blob/master/gocd/api/pipeline.py#L122

        :warning: Replace this with whatever is the official way as soon as gocd#990 is fixed.
        https://github.com/gocd/gocd/issues/990

        :param name: name of the pipeline.
        :param materials: material revisions to use.
        :param variables: environment variables to set.
        :param secure_variables: secure environment variables to set.
        :param backoff: time to wait before checking for new instance.
        :param max_tries: maximum tries to do.
        :return: possible triggered instance of pipeline.
        :rtype: yagocd.resources.pipeline.PipelineInstance
        """
        last_instance = self.last(name)
        if last_instance:
            last_run_counter = last_instance.data.counter
        else:
            last_run_counter = -1

        self.schedule(name=name, materials=materials, variables=variables, secure_variables=secure_variables)

        while max_tries > 0:
            candidate_instance = self.last(name)
            if candidate_instance and candidate_instance.data.counter > last_run_counter:
                return candidate_instance

            time.sleep(backoff)
            max_tries -= 1


class PipelineEntity(Base):
    """
    Class for the pipeline entity, which describes pipeline itself.
    Executing ``history`` will return pipeline instances.
    """

    def __init__(self, session, data, group=None):
        super(PipelineEntity, self).__init__(session, data)
        self._group = group
        self._predecessors = list()
        self._descendants = list()

        self._pipeline = PipelineManager(session=session)

    @property
    def group(self):
        """
        Name of the group pipeline belongs to.
        :return: group name.
        """
        return self._group

    @staticmethod
    def graph_depth_walk(root_nodes, near_nodes):

        visited = set()
        to_crawl = deque(root_nodes)
        while to_crawl:
            current = to_crawl.popleft()
            if current in visited:
                continue
            visited.add(current)
            node_children = set(near_nodes(current))
            to_crawl.extend(node_children - visited)
        return list(visited)

    def get_predecessors(self, transitive=False):
        """
        Property for getting predecessors (parents) of current pipeline.
        This property automatically populates from API call

        :return: list of :class:`yagocd.resources.pipeline.PipelineEntity`.
        :rtype: list of yagocd.resources.pipeline.PipelineEntity
        """
        result = self._predecessors
        if transitive:
            return self.graph_depth_walk(result, lambda v: v.predecessors)
        return result

    def set_predecessors(self, value):
        self._predecessors = value

    predecessors = property(get_predecessors, set_predecessors)

    def get_descendants(self, transitive=False):
        """
        Property for getting descendants (children) of current pipeline.
        It's calculated by :meth:`yagocd.resources.pipeline.PipelineManager#tie_descendants` method during listing of
        all pipelines.

        :return: list of :class:`yagocd.resources.pipeline.PipelineEntity`.
        :rtype: list of yagocd.resources.pipeline.PipelineEntity
        """
        result = self._descendants
        if transitive:
            return self.graph_depth_walk(result, lambda v: v.descendants)
        return result

    def set_descendants(self, value):
        self._descendants = value

    descendants = property(get_descendants, set_descendants)

    @staticmethod
    def get_url(server_url, pipeline_name):
        """
        Returns url for accessing pipeline entity.
        """
        return "{server_url}/go/tab/pipeline/history/{pipeline_name}".format(
            server_url=server_url,
            pipeline_name=pipeline_name
        )

    @property
    def url(self):
        """
        Returns url for accessing pipeline entity.
        """
        return self.get_url(server_url=self._session.server_url, pipeline_name=self.data.name)

    def history(self, offset=0):
        """
        The pipeline history allows users to list pipeline instances.

        :param offset: number of pipeline instances to be skipped.
        :return: an array of pipeline instances :class:`yagocd.resources.pipeline.PipelineInstance`.
        :rtype: list of yagocd.resources.pipeline.PipelineInstance
        """
        return self._pipeline.history(name=self.data.name, offset=offset)

    def full_history(self):
        """
        Method for accessing full history of specific pipeline.

        It yields each instance and after one chunk is over moves to the next one.
        :return: an array of pipeline instances :class:`yagocd.resources.pipeline.PipelineInstance`.
        :rtype: list of yagocd.resources.pipeline.PipelineInstance
        """
        return self._pipeline.full_history(name=self.data.name)

    def last(self):
        """
        Get last pipeline instance.

        :rtype: yagocd.resources.pipeline.PipelineInstance
        """
        return self._pipeline.last(name=self.data.name)

    def get(self, counter=0):
        """
        Gets pipeline instance object.

        :param counter pipeline counter:
        :return: A pipeline instance object :class:`yagocd.resources.pipeline.PipelineInstance`.
        :rtype: yagocd.resources.pipeline.PipelineInstance
        """
        return self._pipeline.get(name=self.data.name, counter=counter)

    def status(self):
        """
        The pipeline status allows users to check if the pipeline is paused, locked and schedulable.

        :return: JSON containing information about pipeline state, wrapped in EasyDict class.
        """
        return self._pipeline.status(name=self.data.name)

    def pause(self, cause):
        """
        Pause the current pipeline.

        :param cause: reason for pausing the pipeline.
        """
        self._pipeline.pause(name=self.data.name, cause=cause)

    def unpause(self):
        """
        Unpause the specified pipeline.
        """
        self._pipeline.unpause(name=self.data.name)

    def release_lock(self):
        """
        Release a lock on a pipeline so that you can start up a new instance
        without having to wait for the earlier instance to finish.

        :return: a text confirmation.
        """
        return self._pipeline.release_lock(name=self.data.name)

    def schedule(self, materials=None, variables=None, secure_variables=None):
        """
        Scheduling allows user to trigger a specific pipeline.

        :param materials: material revisions to use.
        :param variables: environment variables to set.
        :param secure_variables: secure environment variables to set.
        :return: a text confirmation.
        """
        return self._pipeline.schedule(
            name=self.data.name,
            materials=materials,
            variables=variables,
            secure_variables=secure_variables
        )

    def schedule_with_instance(
        self,
        materials=None,
        variables=None,
        secure_variables=None,
        backoff=0.5,
        max_tries=20
    ):
        """
        Schedule pipeline and return instance.
        Credits of implementation comes to `gaqzi`:
        https://github.com/gaqzi/py-gocd/blob/master/gocd/api/pipeline.py#L122

        :warning: Replace this with whatever is the official way as soon as gocd#990 is fixed.
        https://github.com/gocd/gocd/issues/990

        :param materials: material revisions to use.
        :param variables: environment variables to set.
        :param secure_variables: secure environment variables to set.
        :param backoff: time to wait before checking for new instance.
        :param max_tries: maximum tries to do.
        :return: possible triggered instance of pipeline.
        :rtype: yagocd.resources.pipeline.PipelineInstance
        """
        return self._pipeline.schedule_with_instance(
            name=self.data.name,
            materials=materials,
            variables=variables,
            secure_variables=secure_variables,
            backoff=backoff,
            max_tries=max_tries
        )


class PipelineInstance(Base):
    """
    Pipeline instance represents concrete execution of specific pipeline.
    """

    @property
    def url(self):
        """
        Returns url for accessing pipeline instance.
        """
        return "{server_url}/go/pipelines/value_stream_map/{pipeline_name}/{pipeline_counter}".format(
            server_url=self._session.server_url,
            pipeline_name=self.data.name,
            pipeline_counter=self.data.counter
        )

    @property
    def pipeline_url(self):
        """
        Returns url for accessing pipeline entity.
        """
        return PipelineEntity.get_url(server_url=self._session.server_url, pipeline_name=self.data.name)

    def stages(self):
        """
        Method for getting stages from pipeline instance.

        :return: arrays of stages
        :rtype: list of yagocd.resources.stage.StageInstance
        """
        stages = list()
        for data in self.data.stages:
            stages.append(StageInstance(session=self._session, data=data, pipeline=self))

        return stages



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import copy
import json

from six import string_types

from yagocd.client import Yagocd
from yagocd.session import Session
from yagocd.resources import agent, job

import mock
import pytest


class BaseTestAgentManager(object):
    @pytest.fixture()
    def session(self):
        return Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)

    @pytest.fixture()
    def manager(self, session):
        return agent.AgentManager(session=session)


class TestListAsList(BaseTestAgentManager):
    def test_list_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list") as cass:
            manager.list()
            assert cass.requests[0].path == '/go/api/agents'

    def test_list_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list") as cass:
            manager.list()
            assert cass.requests[0].method == 'GET'

    def test_list_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list") as cass:
            manager.list()
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v2+json'

    def test_list_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list") as cass:
            manager.list()
            assert cass.responses[0]['status']['code'] == 200

    def test_list_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list"):
            result = manager.list()
            assert isinstance(result, list)

    def test_list_is_not_empty(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list"):
            result = manager.list()
            assert len(result) > 0

    def test_list_returns_agent_entities(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list"):
            result = manager.list()
            assert all(isinstance(i, agent.AgentEntity) for i in result)


class TestListAsDict(BaseTestAgentManager):
    @pytest.fixture()
    def manager_build_go_cd(self):
        options = copy.deepcopy(Yagocd.DEFAULT_OPTIONS)
        options['server'] = 'https://build.go.cd/go'
        return agent.AgentManager(
            session=Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)
        )

    def test_list_request_url(self, manager_build_go_cd, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_dict") as cass:
            manager_build_go_cd.list()
            assert cass.requests[0].path == '/go/api/agents'

    def test_list_request_method(self, manager_build_go_cd, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_dict") as cass:
            manager_build_go_cd.list()
            assert cass.requests[0].method == 'GET'

    def test_list_request_accept_headers(self, manager_build_go_cd, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_dict") as cass:
            manager_build_go_cd.list()
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v2+json'

    def test_list_response_code(self, manager_build_go_cd, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_dict") as cass:
            manager_build_go_cd.list()
            assert cass.responses[0]['status']['code'] == 200

    def test_list_return_type(self, manager_build_go_cd, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_dict"):
            result = manager_build_go_cd.list()
            assert isinstance(result, list)

    def test_list_is_not_empty(self, manager_build_go_cd, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_dict"):
            result = manager_build_go_cd.list()
            assert len(result) > 0

    def test_list_returns_agent_entities(self, manager_build_go_cd, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_dict"):
            result = manager_build_go_cd.list()
            assert all(isinstance(i, agent.AgentEntity) for i in result)


class TestDict(BaseTestAgentManager):
    @mock.patch('yagocd.resources.agent.AgentManager.list')
    def test_dict_calls_list(self, mock_list, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list"):
            manager.dict()
            mock_list.assert_called()

    def test_dict_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list"):
            result = manager.dict()
            assert isinstance(result, dict)

    def test_dict_returns_str_as_keys(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list"):
            result = manager.dict()
            assert all(isinstance(i, string_types) for i in result.keys())

    def test_dict_returns_agent_entities_as_values(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list"):
            result = manager.dict()
            assert all(isinstance(i, agent.AgentEntity) for i in result.values())

    def test_dict_get_by_key(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_list_as_list"):
            uuid = '68e5d48c-753a-4395-a79c-1cb22d77a12f'
            result = manager.dict().get(uuid)
            fixture = json.load(open('tests/fixtures/resources/agent/{uuid}-from-list.json'.format(uuid=uuid)))
            assert dict(result.data) == fixture


class TestGet(BaseTestAgentManager):
    UUID = '68e5d48c-753a-4395-a79c-1cb22d77a12f'

    def test_get_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_get") as cass:
            manager.get(self.UUID)
            assert cass.requests[0].path == '/go/api/agents/{uuid}'.format(uuid=self.UUID)

    def test_get_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_get") as cass:
            manager.get(self.UUID)
            assert cass.requests[0].method == 'GET'

    def test_get_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_get") as cass:
            manager.get(self.UUID)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_get_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_get") as cass:
            manager.get(self.UUID)
            assert cass.responses[0]['status']['code'] == 200

    def test_get_returns_instance_of_agent_entity(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_get"):
            result = manager.get(self.UUID)
            assert isinstance(result, agent.AgentEntity)


class TestUpdate(BaseTestAgentManager):
    UUID = '68e5d48c-753a-4395-a79c-1cb22d77a12f'
    UPD_CFG = {'hostname': 'foo-bar'}

    def test_update_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_update") as cass:
            manager.update(self.UUID, self.UPD_CFG)
            assert cass.requests[0].path == '/go/api/agents/{uuid}'.format(uuid=self.UUID)

    def test_update_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_update") as cass:
            manager.update(self.UUID, self.UPD_CFG)
            assert cass.requests[0].method == 'PATCH'

    def test_update_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_update") as cass:
            manager.update(self.UUID, self.UPD_CFG)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_update_request_content_type_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_update") as cass:
            manager.update(self.UUID, self.UPD_CFG)
            assert cass.requests[0].headers['content-type'] == 'application/json'

    def test_update_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_update") as cass:
            manager.update(self.UUID, self.UPD_CFG)
            assert cass.responses[0]['status']['code'] == 200

    def test_update_returns_instance_of_agent_entity(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_update"):
            result = manager.update(self.UUID, self.UPD_CFG)
            assert isinstance(result, agent.AgentEntity)

    def test_update_returns_updated_agent(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_update"):
            result = manager.update(self.UUID, self.UPD_CFG)
            assert result.data.hostname == self.UPD_CFG['hostname']


class TestDelete(BaseTestAgentManager):
    UUID = '68e5d48c-753a-4395-a79c-1cb22d77a12f'

    def test_delete_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_delete") as cass:
            manager.delete(self.UUID)
            assert cass.requests[0].path == '/go/api/agents/{uuid}'.format(uuid=self.UUID)

    def test_delete_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_delete") as cass:
            manager.delete(self.UUID)
            assert cass.requests[0].method == 'DELETE'

    def test_delete_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_delete") as cass:
            manager.delete(self.UUID)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v1+json'

    def test_delete_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_delete") as cass:
            manager.delete(self.UUID)
            assert cass.responses[0]['status']['code'] == 200

    def test_delete_return_message(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_delete") as cass:
            manager.delete(self.UUID)
            assert b'Deleted 1 agent(s)' in cass.responses[0]['body']['string']


class TestJobHistory(BaseTestAgentManager):
    UUID = '68e5d48c-753a-4395-a79c-1cb22d77a12f'

    def test_job_history_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_job_history") as cass:
            manager.job_history(self.UUID)
            assert cass.requests[0].path == '/go/api/agents/{uuid}/job_run_history/{offset}'.format(
                uuid=self.UUID, offset=0
            )

    def test_job_history_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_job_history") as cass:
            manager.job_history(self.UUID)
            assert cass.requests[0].method == 'GET'

    def test_job_history_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_job_history") as cass:
            manager.job_history(self.UUID)
            assert cass.requests[0].headers['accept'] == 'application/vnd.go.cd.v2+json'

    def test_job_history_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_job_history") as cass:
            manager.job_history(self.UUID)
            assert cass.responses[0]['status']['code'] == 200

    def test_job_history_return_type(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_job_history"):
            result = manager.job_history(self.UUID)
            assert isinstance(result, list)

    def test_job_history_returns_job_instances(self, manager, my_vcr):
        with my_vcr.use_cassette("agent/agent_job_history"):
            result = manager.job_history(self.UUID)
            assert all(isinstance(i, job.JobInstance) for i in result)





#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd import Yagocd
from yagocd.resources import pipeline, stage, job, artifact, property as prop
from yagocd.session import Session

import mock
import pytest


class TestJobInstance(object):
    PIPELINE_NAME = 'Consumer_Website'
    PIPELINE_COUNTER = 29
    STAGE_NAME = 'Commit'
    STAGE_COUNTER = '1'
    JOB_NAME = 'build'
    URL = 'http://localhost:8153/go/tab/build/detail/Consumer_Website/29/Commit/1/build'

    @pytest.fixture()
    def session(self):
        return Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)

    @pytest.fixture()
    def job_instance_from_pipeline(self, my_vcr, session):
        with my_vcr.use_cassette("job/job_instance_from_pipeline"):
            return pipeline.PipelineManager(session).find(self.PIPELINE_NAME).history()[0].stages()[0].jobs()[0]

    @pytest.fixture()
    def job_instance_from_stage(self, my_vcr, session):
        with my_vcr.use_cassette("job/job_instance_from_stage"):
            return stage.StageManager(session).get(
                self.PIPELINE_NAME,
                self.PIPELINE_COUNTER,
                self.STAGE_NAME,
                self.STAGE_COUNTER
            ).jobs()[0]

    @pytest.fixture()
    def job_instance_from_job_manager(self, my_vcr, session):
        with my_vcr.use_cassette("job/job_instance_from_job_manager"):
            return job.JobManager(session).history(self.PIPELINE_NAME, self.STAGE_NAME, self.JOB_NAME)[0]

    @pytest.mark.parametrize("job_fixture_name", [
        'job_instance_from_pipeline',
        'job_instance_from_stage',
        'job_instance_from_job_manager',
    ])
    def test_pipeline_name(self, job_fixture_name, request):
        job_fixture = request.getfuncargvalue(job_fixture_name)
        assert job_fixture.pipeline_name == self.PIPELINE_NAME, "Fixture: {}".format(job_fixture_name)

    @pytest.mark.parametrize("job_fixture_name", [
        'job_instance_from_pipeline',
        'job_instance_from_stage',
        'job_instance_from_job_manager',
    ])
    def test_pipeline_counter(self, job_fixture_name, request):
        job_fixture = request.getfuncargvalue(job_fixture_name)
        assert job_fixture.pipeline_counter == self.PIPELINE_COUNTER, "Fixture: {}".format(job_fixture_name)

    @pytest.mark.parametrize("job_fixture_name", [
        'job_instance_from_pipeline',
        'job_instance_from_stage',
        'job_instance_from_job_manager',
    ])
    def test_stage_name(self, job_fixture_name, request):
        job_fixture = request.getfuncargvalue(job_fixture_name)
        assert job_fixture.stage_name == self.STAGE_NAME, "Fixture: {}".format(job_fixture_name)

    @pytest.mark.parametrize("job_fixture_name", [
        'job_instance_from_pipeline',
        'job_instance_from_stage',
        'job_instance_from_job_manager',
    ])
    def test_stage_counter(self, job_fixture_name, request):
        job_fixture = request.getfuncargvalue(job_fixture_name)
        assert str(job_fixture.stage_counter) == self.STAGE_COUNTER, "Fixture: {}".format(job_fixture_name)

    @pytest.mark.parametrize("job_fixture_name", [
        'job_instance_from_pipeline',
        'job_instance_from_stage',
        'job_instance_from_job_manager',
    ])
    def test_url(self, job_fixture_name, request):
        job_fixture = request.getfuncargvalue(job_fixture_name)
        assert job_fixture.url == self.URL, "Fixture: {}".format(job_fixture_name)

    @pytest.mark.parametrize("job_fixture_name", [
        'job_instance_from_pipeline',
        'job_instance_from_stage',
    ])
    def test_stage_provided(self, job_fixture_name, request):
        job_fixture = request.getfuncargvalue(job_fixture_name)
        assert job_fixture.stage is not None, "Fixture: {}".format(job_fixture_name)
        assert isinstance(job_fixture.stage, stage.StageInstance), "Fixture: {}".format(job_fixture_name)

    @pytest.mark.parametrize("job_fixture_name", [
        'job_instance_from_job_manager',
    ])
    def test_stage_is_empty(self, job_fixture_name, request):
        job_fixture = request.getfuncargvalue(job_fixture_name)
        assert job_fixture.stage is None, "Fixture: {}".format(job_fixture_name)

    @pytest.mark.parametrize("job_fixture_name", [
        'job_instance_from_pipeline',
        'job_instance_from_stage',
        'job_instance_from_job_manager',
    ])
    def test_artifact(self, job_fixture_name, request):
        job_fixture = request.getfuncargvalue(job_fixture_name)
        assert job_fixture.artifact is not None, "Fixture: {}".format(job_fixture_name)
        assert isinstance(job_fixture.artifact, artifact.ArtifactManager), "Fixture: {}".format(job_fixture_name)

    @pytest.mark.parametrize("job_fixture_name", [
        'job_instance_from_pipeline',
        'job_instance_from_stage',
        'job_instance_from_job_manager',
    ])
    def test_property(self, job_fixture_name, request):
        job_fixture = request.getfuncargvalue(job_fixture_name)
        assert job_fixture.prop is not None, "Fixture: {}".format(job_fixture_name)
        assert isinstance(job_fixture.prop, prop.PropertyManager), "Fixture: {}".format(job_fixture_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import hashlib

from yagocd.client import Yagocd
from yagocd.session import Session
from yagocd.resources import pipeline

import mock
import pytest
from requests import HTTPError


class BaseTestPipelineManager(object):
    @staticmethod
    def get_suffix(*args):
        m = hashlib.md5()
        m.update('|'.join([str(x) for x in args]).encode('utf-8'))
        m.hexdigest()
        return m.hexdigest()[:8]

    @pytest.fixture()
    def session(self):
        return Session(auth=None, options=Yagocd.DEFAULT_OPTIONS)

    @pytest.fixture()
    def manager(self, session):
        return pipeline.PipelineManager(session=session)


class TestTieDescendants(BaseTestPipelineManager):
    def test_tie_descendants(self, session, manager):
        child = pipeline.PipelineEntity(
            session=session,
            data={'name': 'child1', 'materials': {}}
        )

        parent = pipeline.PipelineEntity(
            session=session,
            data={'name': 'parent1', 'materials': [{'description': 'child1', 'type': 'Pipeline'}]}
        )

        pipelines = [child, parent]

        manager.tie_pipelines(pipelines)
        assert child.descendants == [parent]


class TestList(BaseTestPipelineManager):
    def test_list_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list") as cass:
            manager.list()
            assert cass.requests[0].path == '/go/api/config/pipeline_groups'

    def test_list_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list") as cass:
            manager.list()
            assert cass.requests[0].method == 'GET'

    def test_list_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list") as cass:
            manager.list()
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_list_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list") as cass:
            manager.list()
            assert cass.responses[0]['status']['code'] == 200

    def test_list_is_not_empty(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list"):
            result = manager.list()
            assert len(result) > 0

    def test_elements_in_list_are_pipelines_entities(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list"):
            result = manager.list()
            assert all(isinstance(i, pipeline.PipelineEntity) for i in result)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.tie_pipelines')
    def test_tie_descendants_is_called(self, mock_tie_pipelines, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list"):
            manager.list()
            mock_tie_pipelines.assert_called()


class TestFind(BaseTestPipelineManager):
    @mock.patch('yagocd.resources.pipeline.PipelineManager.list')
    def test_list_is_called(self, mock_list, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list"):
            name = 'Production_Services'
            manager.find(name)
            mock_list.assert_called()

    def test_find_non_existing(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list"):
            result = manager.find('This_Pipeline_Doesnt_Exists')
            assert result is None

    def test_find_returns_pipeline_entity(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list"):
            name = 'Production_Services'
            result = manager.find(name)
            assert isinstance(result, pipeline.PipelineEntity)

    def test_find_returns_entity_with_same_name(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pipeline_list"):
            name = 'Production_Services'
            result = manager.find(name)
            assert result.data.name == name


class TestHistory(BaseTestPipelineManager):
    def test_history_non_existing(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/history_non_existing"):
            with pytest.raises(HTTPError):
                manager.history("pipeline_non_existing")

    def test_history_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/history_Consumer_Website") as cass:
            name = "Consumer_Website"
            manager.history(name)
            assert cass.requests[0].path == '/go/api/pipelines/{name}/history/{offset}'.format(
                name=name, offset=0
            )

    def test_history_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/history_Consumer_Website") as cass:
            name = "Consumer_Website"
            manager.history(name)
            assert cass.requests[0].method == 'GET'

    def test_history_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/history_Consumer_Website") as cass:
            name = "Consumer_Website"
            manager.history(name)
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_history_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/history_Consumer_Website") as cass:
            name = "Consumer_Website"
            manager.history(name)
            assert cass.responses[0]['status']['code'] == 200

    def test_history(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/history_Consumer_Website"):
            name = "Consumer_Website"
            result = manager.history(name)

            assert all(isinstance(i, pipeline.PipelineInstance) for i in result)
            assert all(i.data.name == name for i in result)


class TestFullHistory(BaseTestPipelineManager):
    @mock.patch('yagocd.resources.pipeline.PipelineManager.history')
    def test_history_is_called(self, history_mock, manager, my_vcr):
        history_mock.side_effect = [['foo', 'bar', 'baz'], []]

        name = "Consumer_Website"
        list(manager.full_history(name))

        calls = [mock.call(name, 0), mock.call(name, 3)]
        history_mock.assert_has_calls(calls)


class TestLast(BaseTestPipelineManager):
    @mock.patch('yagocd.resources.pipeline.PipelineManager.history')
    def test_history_is_called(self, history_mock, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/history_Consumer_Website"):
            name = "Consumer_Website"
            manager.last(name)
            history_mock.assert_called_with(name=name)

    @mock.patch('yagocd.resources.pipeline.PipelineManager.history')
    def test_last_return_last(self, history_mock, manager, my_vcr):
        history_mock.return_value = ['foo', 'bar', 'baz']
        with my_vcr.use_cassette("pipeline/history_Consumer_Website"):
            name = "Consumer_Website"
            assert manager.last(name) == 'foo'


class TestGet(BaseTestPipelineManager):
    def test_get_non_existing(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/get_non_existing"):
            with pytest.raises(HTTPError):
                manager.get("pipeline_instance_non_existing", 1)

    def test_get_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/get_Consumer_Website") as cass:
            name = "Consumer_Website"
            counter = 2
            manager.get(name, counter)
            assert cass.requests[0].path == '/go/api/pipelines/{name}/instance/{counter}'.format(
                name=name, counter=counter
            )

    def test_get_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/get_Consumer_Website") as cass:
            name = "Consumer_Website"
            manager.get(name, 2)
            assert cass.requests[0].method == 'GET'

    def test_get_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/get_Consumer_Website") as cass:
            name = "Consumer_Website"
            manager.get(name, 2)
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_get_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/get_Consumer_Website") as cass:
            name = "Consumer_Website"
            manager.get(name, 2)
            assert cass.responses[0]['status']['code'] == 200

    def test_get(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/get_Consumer_Website"):
            name = "Consumer_Website"
            result = manager.get(name, 2)

            assert isinstance(result, pipeline.PipelineInstance)
            assert result.data.name == name

    def test_status(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/status_Consumer_Website"):
            name = "Consumer_Website"
            result = manager.status(name)

            assert result == {'paused': False, 'schedulable': True, 'locked': False}


class TestPause(BaseTestPipelineManager):
    def test_pause(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/pause_Consumer_Website"):
            name = "Consumer_Website"
            reason = 'Test pause reason'
            manager.unpause(name)

            result = manager.pause(name, reason)

            assert result is None
            assert manager.status(name) == {'paused': True, 'schedulable': False, 'locked': False}

            manager.unpause(name)


class TestUnPause(BaseTestPipelineManager):
    def test_unpause(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/unpause_Consumer_Website"):
            name = "Consumer_Website"
            manager.pause(name, '')

            result = manager.unpause(name)
            assert result is None

            assert manager.status(name) == {'paused': False, 'schedulable': True, 'locked': False}


class TestReleaseLock(BaseTestPipelineManager):
    def test_release_lock_request_url(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/release_lock") as cass:
            name = "Deploy_UAT"
            manager.release_lock(name)
            assert cass.requests[0].path == '/go/api/pipelines/{name}/releaseLock'.format(
                name=name
            )

    def test_release_lock_request_method(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/release_lock") as cass:
            name = "Deploy_UAT"
            manager.release_lock(name)
            assert cass.requests[0].method == 'POST'

    def test_release_lock_request_accept_headers(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/release_lock") as cass:
            name = "Deploy_UAT"
            manager.release_lock(name)
            assert cass.requests[0].headers['accept'] == 'application/json'

    def test_release_lock_response_code(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/release_lock") as cass:
            name = "Deploy_UAT"
            manager.release_lock(name)
            assert cass.responses[0]['status']['code'] == 200

    def test_release_lock_return_value(self, manager, my_vcr):
        with my_vcr.use_cassette("pipeline/release_lock"):
            name = "Deploy_UAT"
            result = manager.release_lock(name)
            assert result == 'pipeline lock released for {0}\n'.format(name)


class TestSchedule(BaseTestPipelineManager):
    NAME = "Shared_Services"

    @pytest.mark.parametrize("variables", [None, {'MY_VARIABLE': 'some value'}])
    @pytest.mark.parametrize("secure_variables", [None, {'MY_SECRET_VARIABLE': 'secret variable'}])
    def test_schedule_request_url(self, manager, my_vcr, variables, secure_variables):
        suffix = self.get_suffix(variables, secure_variables)
        with my_vcr.use_cassette("pipeline/schedule-{0}".format(suffix)) as cass:
            manager.schedule(name=self.NAME, variables=variables, secure_variables=secure_variables)
            assert cass.requests[0].path == '/go/api/pipelines/{name}/schedule'.format(
                name=self.NAME
            )

    @pytest.mark.parametrize("variables", [None, {'MY_VARIABLE': 'some value'}])
    @pytest.mark.parametrize("secure_variables", [None, {'MY_SECRET_VARIABLE': 'secret variable'}])
    def test_schedule_request_method(self, manager, my_vcr, variables, secure_variables):
        suffix = self.get_suffix(variables, secure_variables)
        with my_vcr.use_cassette("pipeline/schedule-{0}".format(suffix)) as cass:
            manager.schedule(name=self.NAME, variables=variables, secure_variables=secure_variables)
            assert cass.requests[0].method == 'POST'

    @pytest.mark.parametrize("variables", [None, {'MY_VARIABLE': 'some value'}])
    @pytest.mark.parametrize("secure_variables", [None, {'MY_SECRET_VARIABLE': 'secret variable'}])
    def test_schedule_request_accept_headers(self, manager, my_vcr, variables, secure_variables):
        suffix = self.get_suffix(variables, secure_variables)
        with my_vcr.use_cassette("pipeline/schedule-{0}".format(suffix)) as cass:
            manager.schedule(name=self.NAME, variables=variables, secure_variables=secure_variables)
            assert cass.requests[0].headers['accept'] == 'application/json'
            assert cass.requests[0].headers['content-type'] == 'application/json'

    @pytest.mark.parametrize("variables", [None, {'MY_VARIABLE': 'some value'}])
    @pytest.mark.parametrize("secure_variables", [None, {'MY_SECRET_VARIABLE': 'secret variable'}])
    def test_schedule_response_code(self, manager, my_vcr, variables, secure_variables):
        suffix = self.get_suffix(variables, secure_variables)
        with my_vcr.use_cassette("pipeline/schedule-{0}".format(suffix)) as cass:
            manager.schedule(name=self.NAME, variables=variables, secure_variables=secure_variables)
            assert cass.responses[0]['status']['code'] == 202

    @pytest.mark.parametrize("variables", [None, {'MY_VARIABLE': 'some value'}])
    @pytest.mark.parametrize("secure_variables", [None, {'MY_SECRET_VARIABLE': 'secret variable'}])
    def test_schedule_return_value(self, manager, my_vcr, variables, secure_variables):
        suffix = self.get_suffix(variables, secure_variables)
        with my_vcr.use_cassette("pipeline/schedule-{0}".format(suffix)):
            result = manager.schedule(name=self.NAME, variables=variables, secure_variables=secure_variables)
            assert result == 'Request to schedule pipeline {0} accepted\n'.format(self.NAME)


class TestScheduleWithInstance(BaseTestPipelineManager):
    NAME = "Shared_Services"

    @pytest.mark.parametrize("variables", [None, {'MY_VARIABLE': 'some value'}])
    @pytest.mark.parametrize("secure_variables", [None, {'MY_SECRET_VARIABLE': 'secret variable'}])
    @mock.patch('yagocd.resources.pipeline.PipelineManager.schedule')
    def test_schedule_is_called(self, schedule_mock, manager, my_vcr, variables, secure_variables):
        suffix = self.get_suffix(variables, secure_variables)
        with my_vcr.use_cassette("pipeline/schedule_with_instance-{0}".format(suffix)):
            manager.schedule_with_instance(name=self.NAME, variables=variables, secure_variables=secure_variables,
                                           backoff=0)
            schedule_mock.assert_called()

    @pytest.mark.parametrize("variables", [None, {'MY_VARIABLE': 'some value'}])
    @pytest.mark.parametrize("secure_variables", [None, {'MY_SECRET_VARIABLE': 'secret variable'}])
    def test_return_value(self, manager, my_vcr, variables, secure_variables):
        suffix = self.get_suffix(variables, secure_variables)
        with my_vcr.use_cassette("pipeline/schedule_with_instance-{0}".format(suffix)):
            result = manager.schedule_with_instance(name=self.NAME, variables=variables,
                                                    secure_variables=secure_variables, backoff=0)
            assert isinstance(result, pipeline.PipelineInstance)

    @pytest.mark.parametrize("variables", [None, {'MY_VARIABLE': 'some value'}])
    @pytest.mark.parametrize("secure_variables", [None, {'MY_SECRET_VARIABLE': 'secret variable'}])
    @mock.patch('yagocd.resources.pipeline.PipelineManager.last')
    def test_empty_history(self, last_mock, session, manager, my_vcr, variables, secure_variables):
        suffix = self.get_suffix(variables, secure_variables)
        last_mock.side_effect = [None, [], pipeline.PipelineInstance(session, dict(counter=3))]
        with my_vcr.use_cassette("pipeline/schedule_with_instance_custom_history-{0}".format(suffix)):
            result = manager.schedule_with_instance(name=self.NAME, variables=variables,
                                                    secure_variables=secure_variables, backoff=0)
            assert isinstance(result, pipeline.PipelineInstance)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import json

from yagocd.resources import BaseManager, Base
from yagocd.resources.job import JobInstance


class AgentManager(BaseManager):
    """
    The agents API allows users with administrator role to manage agents.

    :warning: Please note that this API requires using v2 of the API using `Accept: application/vnd.go.cd.v2+json`
    """

    # Using this header is actually impossible, as it doesn't work as documented in most cases...
    ACCEPT_HEADER = 'application/vnd.go.cd.v2+json'

    def list(self):
        """
        Lists all available agents, these are agents that are present in the
        <agents/> tag inside cruise-config.xml and also agents that are in
        Pending state awaiting registration.

        :return: an array of agents.
        :rtype: list of yagocd.resources.agent.AgentEntity
        """
        response = self._session.get(
            path='{base_api}/agents'.format(base_api=self.base_api),
            headers={'Accept': self.ACCEPT_HEADER},
        )

        agents = list()
        # Depending on Go version, return value would be either list of dict.
        # Support both cases here.
        json_response = response.json()
        if isinstance(json_response, list):
            agents_json = json_response
        elif isinstance(json_response, dict):
            agents_json = json_response.get('_embedded', {}).get('agents', {})
        else:
            raise ValueError("Expected response to be in [list, dict], but '{}' found!".format(json_response))

        for data in agents_json:
            agents.append(AgentEntity(session=self._session, data=data))

        return agents

    def dict(self):
        """
        Wrapper for `list()` method, that transforms founded agents to
        dictionary by `uuid` key.

        :return: dictionary of agents with `uuid` as a key and agent as a value.
        :rtype: dict[str, yagocd.resources.agent.AgentEntity]
        """
        agents = self.list()
        result = dict()
        for agent in agents:
            result[agent.data.uuid] = agent

        return result

    def get(self, uuid):
        """
        Gets an agent by its unique identifier (uuid).

        :param uuid: uuid of the agent
        :return: Agent entity.
        :rtype: yagocd.resources.agent.AgentEntity
        """
        response = self._session.get(
            path='{base_api}/agents/{uuid}'.format(
                base_api=self.base_api,
                uuid=uuid,
            ),
            # and again, WTF?!!
            # This depends on Go version: v2 works for later version of Go (16), but doesn't for earlier (15).
            # And because there is no good way to get version of the Go server (WTF?!), just use v1 header for now.
            # headers={'Accept': self.ACCEPT_HEADER},
        )

        return AgentEntity(session=self._session, data=response.json())

    def update(self, uuid, config):
        """
        Update some attributes of an agent.

        :param uuid: uuid of the agent
        :param config: dictionary of parameters for update
        :return: Agent entity.
        :rtype: yagocd.resources.agent.AgentEntity
        """
        response = self._session.patch(
            path='{base_api}/agents/{uuid}'.format(
                base_api=self.base_api,
                uuid=uuid,
            ),
            data=json.dumps(config),
            headers={
                # 'Accept': self.ACCEPT_HEADER,  # WTF?!!
                'Content-Type': 'application/json'
            },
        )

        return AgentEntity(session=self._session, data=response.json())

    def delete(self, uuid):
        """
        Deletes an agent.

        :param uuid: uuid of the agent.
        :return: a message confirmation if the agent was deleted.
        """
        response = self._session.delete(
            path='{base_api}/agents/{uuid}'.format(
                base_api=self.base_api,
                uuid=uuid,
            ),
            # headers={'Accept': self.ACCEPT_HEADER},  # WTF?!!
        )

        return response.json().get('message')

    def job_history(self, uuid, offset=0):
        """
        Lists the jobs that have executed on an agent.

        :param uuid: uuid of the agent.
        :param offset: number of jobs to be skipped.
        :return: an array of :class:`yagocd.resources.job.JobInstance` along with the job transitions.
        :rtype: list of yagocd.resources.job.JobInstance
        """
        response = self._session.get(
            path='{base_api}/agents/{uuid}/job_run_history/{offset}'.format(
                base_api=self.base_api,
                uuid=uuid,
                offset=offset
            ),
            headers={'Accept': self.ACCEPT_HEADER},
        )

        jobs = list()
        for data in response.json()['jobs']:
            jobs.append(JobInstance(session=self._session, data=data, stage=None))
        return jobs


class AgentEntity(Base):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################
import json

from yagocd.resources import pipeline
from yagocd.resources import stage
from yagocd.resources import Base
import pytest


class TestPipelineEntity(object):
    @pytest.fixture()
    def pipeline_instance(self, mock_session):
        data = json.load(open('tests/fixtures/resources/pipeline/pipeline_instance.json'))
        return pipeline.PipelineInstance(session=mock_session, data=data)

    def test_instance_is_not_none(self, pipeline_instance):
        assert pipeline_instance is not None

    def test_is_instance_of_base(self, pipeline_instance):
        assert isinstance(pipeline_instance, Base)

    def test_getting_name(self, pipeline_instance):
        assert pipeline_instance.data.name == 'Shared_Services'

    def test_getting_url(self, pipeline_instance):
        assert pipeline_instance.url == 'http://example.com/go/pipelines/value_stream_map/Shared_Services/2'

    def test_getting_pipeline_url(self, pipeline_instance):
        assert pipeline_instance.pipeline_url == 'http://example.com/go/tab/pipeline/history/Shared_Services'

    def test_stages_are_not_empty(self, pipeline_instance):
        assert len(pipeline_instance.stages()) > 0

    def test_stages_instances(self, pipeline_instance):
        assert all(isinstance(i, stage.StageInstance) for i in pipeline_instance.stages())
